/*************************************************************************/
/*                                                                       */
/*   Program Name :  m5time                                              */
/*                                                                       */
/*     M5time monitors header of Mark-5 file                             */
/*                                                                       */
/*     Enviroment variables                                              */
/*           M5DIR      ---   default directory for Mark5 data           */
/*                                                                       */
/*      Version:  1.21  2007-03-29                                       */
/*                                                                       */
/*  copyright (c) 2003-2007 T.Kondo/NICT(CRL) All Right Reserved         */
/*                                                                       */
/*************************************************************************/
/*

  Mark-5 data sync block monitor

          T.Kondo 2004.11.13
                  2005.06-17 update due to the library restructuring
				  2007.03.28 add '-s nbyte' parameter
				  2007.03.29 add '-s nbyte' parameter
                 

    m5time mk5_filename [options]
    
     where   mk5filename -- Mark 5 data file name
              options (any order)
               -b bitpos  -- pickup bitposition (0- (ntrack-1))
                            (default 0)
               -t[rack] ntrack -- #of bits/word
                            (default 32)
               -n nkai -- # of frames to display (read)
                            0 means all (default 20)
               -vlba      -- file is VLBA Mark5 data

               -s nbyte   -- skip first nbyte

G:\mark5\m5test.dat -b 0 -vlba -t 64
G:\IPVLBI\data\11-K178-1321 -b 0 -vlba -t 32
G:\GSICHECK\K04242\242-0734 -b 2
*/
#define PROGRAM_VERSION "m5time (Ver 1.21  2007-03-29)"


#include "libk5.h"      // common header for IP-VLBI utilities 

#include "libmk5tok5.h"  // library for Mark5 to K5 conversion

/* environment variable for Mark5 data default directory set */
#define M5DIR "M5DIR" // the environment variable for Mark5 data default directory


void param_get_m5time(int argc, char *argv[],
               char *mk5name, 
               int *bitpos, int *ntrack, int *nkai, int *vlba, int *sbyte)
{
    int n;
    char cbuf[80];
    //1st param should be mark5 name
    if (argc>=2){
       strcpy(mk5name,*(argv+1));
    } else {
       mk5name[0]='\0';
    }

    /* check Mark5 file name is acompanied by directory or not */
    get_true_path(mk5name, M5DIR, "./",cbuf);
    strcpy(mk5name,cbuf);

    // initialize parameters
    *bitpos=0;
    *ntrack=32;
    *nkai=20;
    *vlba=0;
	*sbyte=0;

    n=3;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));

        if(strncmp(cbuf,"-b",2)==0){  // bitpos
            n++;
            if(n > argc) break;
            *bitpos = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-t",2)==0){  // ntrack
            n++;
            if(n > argc) break;
            *ntrack = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-n",2)==0){  // nkai
            n++;
            if(n > argc) break;
            *nkai = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-s",2)==0){  // skip nbytes
            n++;
            if(n > argc) break;
            *sbyte = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-v",2)==0){  // VLBA mode
            *vlba=1;
        }
        n++;
    }
}

void print_usage(void)
{
    //printf("m5tok5Rnp  %s\n",M5TOK5RNP_VERSION);
    printf("%s\n",PROGRAM_VERSION);
    printf("Usage: m5time mk5_filename [options]\n");
    printf("\n");
    printf("where   mk5filename -- Mark5 data file name\n");
    printf("       options (any order)\n");
    printf("        -b bitpos  -- pickup bitposition (0- (ntrack-1))\n");
    printf("                      (default 0)\n");
    printf("        -t[rack] ntrack -- #of bits/word\n");
    printf("                          (default 32)\n");
    printf("        -n nkai -- # of frames to display (read)\n");
    printf("                    0 means all frames (default 20)\n");
    printf("                            (default 20)\n");
    printf("        -v[lba]      -- file is VLBA Mark5 data\n");
    printf("        -s nbyte -- skip first nbyte\n");
    printf("                            (default 0)\n");
    printf("   Enviroment variables\n");
    printf("        %s    ---   default directory for Mark5 data\n",M5DIR);
    printf("\n");
}


int read_1frame_r_mugen(mk5_data_t *mk5)  // for all mode
/*
  2007-03-28

  IN
    mk5
     

  Return
     mk5->itime[6]  --  Time at BOFRAME
	 mk5->header[5] --  Header at BOFRAME
	 mk5->idata[20160] -- data stream  1 frame
                       Mark-IV  : 20000
                       VLBA format : 20160


*/
{
       int sync_pat1[]={1,1,1,1,1,1,1,1,0,      //sync pattern including parity bit
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0,
                   1,1,1,1,1,1,1,1,0};
       int sync_pat0[]={1,1,1,1,1,1,1,1,      //sync pattern without parity bit
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1,
                   1,1,1,1,1,1,1,1};
       int *sync_pat;
       int sync[36];
       //int head[180];  // header 180bits including parity
	   int workd[22680];  // to allow VLBA mode 1frame
	   int ndata;

       int idat[2],itime[6];
       int ihead;
       int i,ii,ih, kold, iread, tc, imax, ikai, k, m,ihan;
	   int npos,nstart,isync,bitpos;
	   int icount,nread,ierr;
       int ntrack,parity,nrzm;
	   int vflag;
	   char cw;

	   ierr=0;
       icount=0;
       ntrack=mk5->ntrack;
       parity=mk5->parity;
       nrzm=mk5->nrzm;
       if(parity==1){
          imax=36;
          ihead=180;
          sync_pat=sync_pat1;
       } else {
          imax=32;
          ihead=160;
          sync_pat=sync_pat0;
       }
       tc=-1;

       ikai=0;
       //kold=0;
       kold=mk5->ko;
	   bitpos=mk5->bitpos;
       if(mk5->vlba == 1) {
           ndata=20160;
       } else {
           ndata=20000;
       }
       if(parity == 1) ndata=(ndata*9)/8;
	   //ndata=22500;
	   //ndata=20000;
	   mk5->ndata=0;

       //printf("parity,nrzm,ndata %d %d %d\n",parity,nrzm,ndata);

       //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
       if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm); 
       if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
       if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
       if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
	   if(iread < ndata) {  // eof detection
		   return 0;
	   }
       /* sync detection */
	   isync=0;
	   for(k=0; k<ndata-imax+1; k++){
		  ihan=0;
	      for(i=0; i<imax; i++){
		     if(workd[k+i]!=sync_pat[i]) ihan=1;
		  }
		  if(ihan ==0) {   // sync detected
			  npos=k;
			  isync=1;
			  break;
		  }
	   }
	   vflag=0;
try_again:
	   if(isync ==0) {  // sync not found
	      for(i=0; i<imax-1; i++){
			 k=ndata-imax+1;
		     sync[i]=workd[k+i];
		  }

          //iread=read_ndata_r(mk5->fin, bitpos, ndata, workd, &kold, ntrack, nrzm);
          if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
          if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, ndata, workd, &kold, nrzm);
	      if(iread < ndata) {  // eof detection
		      return 0;
		  }

		  for(k=0; k<imax; k++){
			  sync[imax-1]=workd[k];
		      ihan=0;
	          for(i=0; i<imax; i++){
		         if(workd[k+i]!=sync_pat[i]) ihan=1;
			  } 
		      if(ihan ==0) {   // sync detected
			      npos=k-imax+1;
			      isync=1;
			      break;
			  } 

              for(i=0; i< imax-1; i++){
                  sync[i]=sync[i+1];
              }
		  }
		  // further sync search
		  if(isync==0){//
	         for(k=imax; k<ndata-imax+1; k++){
		        ihan=0;
	            for(i=0; i<imax; i++){
		           if(workd[k+i]!=sync_pat[i]) ihan=1;
				} 
		        if(ihan ==0) {   // sync detected
			       npos=k;
			       isync=1;
			       break;
				} 
			 } 
		  }
	   }

	   if(isync==0){
		   if(vflag==0){
		       printf("Sync not detected in 2 frames! \n");
               printf(" Hit return key for continuing search ");
          //scanf("%s",&cw);  // dummy read
               cw=getchar();  // dummy read to accept just return
			   vflag=1;
		   }
		   goto try_again;
		   //return 0;
	   }
       //printf("Sync detected!\n");

	   // found start point of frame
	   m=0;
	   ii=0;
	   ih=0;
	   nstart=npos-2*imax;
	   if(nstart <0) nstart+=ndata;
	   icount=ndata-nstart;
       //printf("Debug ndata,nstart,icount %d %d %d\n",ndata,nstart,icount);  //debug
	   for(i=0; i<icount; i++){
		   //if(ih<ihead) {
		   //   head[ih]=workd[i+nstart];
		   //   ih++;
		   //}
           if(parity==1){
              if((m % 9) != 8) {     // to remove parity bit
			     mk5->idata[ii]=workd[i+nstart];
			     ii++;
              }
           } else {
			  mk5->idata[ii]=workd[i+nstart];
			  ii++;
           }
		   m++;
	   }
	   nread=ndata-icount;
    //   iread=read_ndata_r(mk5->fin, bitpos, nread, workd, &kold, ntrack, nrzm);
       if(ntrack==8)  iread=read_ndata8_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==16) iread=read_ndata16_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==32) iread=read_ndata32_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
       if(ntrack==64) iread=read_ndata64_r(mk5->fin, bitpos, nread, workd, &kold, nrzm);
	   if(iread < nread) {  // eof detection
	      return 0;
	   } 
	   for(i=0; i<nread; i++){
		   //if(ih<ihead) {
		   //  head[ih]=workd[i];
		   //  ih++;
		   //}
           if(parity==1){
              if((m % 9) != 8) {     // to remove parity bit
			     mk5->idata[ii]=workd[i];
			     ii++;
              }
           } else {
			  mk5->idata[ii]=workd[i];
			  ii++;
           }
		   m++;
	   }
       mk5->ko=kold;
       /*  now idata contains full one frame without parity */
       /* convert header to int data */
       for(i=0; i<5; i++){
           //for(k=0; k<36; k++){
           //for(k=0; k<imax; k++){
           for(k=0; k<32; k++){
              //sync[k]=head[i*imax+k];
              sync[k]=mk5->idata[i*32+k];
           }
           mk5->header[i]=conv2intn(sync,1);
       }
	   //debug
       //for (i=0; i<5; i++) printf("%08x ",rvsint(mk5->header[i]));
	   //printf("\n");

	   idat[0]=rvsint(mk5->header[3]);
	   idat[1]=rvsint(mk5->header[4]);
       //conv2time(mk5->header[3],mk5->itime);
       if(mk5->vlba == 1) {
          conv2time_vlba(idat,itime);
       } else {
          conv2time(idat,itime);
       }
       for(i=0; i<6; i++) mk5->itime[i]=itime[i];
	   mk5->ndata=ii;
       mk5->dtime=3600.0*(double)itime[2]+60.0*(double)itime[3]
           +(double)itime[4]+(double)itime[5]/1000.0;

       //get_trk_from_header(mk5->header, &mk5->aux.trk, &mk5->aux.adid);
       get_trk_from_header_a(mk5->header, &mk5->aux.trk, &mk5->aux.adid,mk5->vlba);
       //parseMK4aux(mk5->header,&mk5->aux);  // when get all aux info
	   return ii;
}

int main(int argc, char *argv[])
{
       int i,n;
       int bitpos,ntrack,vlba,nkai;
	   int sbyte;
       char mk5name[80];
       char program_version[60];
       char* dflt_fname="G:/IPVLBI/data/jiveexp/Wb_test";
      // IPBOARD_T ipb;
       mk5_data_t mk5;    // to suport 2 bit sampling 4 ch and fanout 4 conversion

       if(argc == 1){
          print_usage();
          exit(0);
       }
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   default directory for Mark5 data\n",M5DIR);
          printf("                      ( %s )\n",getenv(M5DIR));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* Program Name Display */
       printf("****************************************************\n");
       printf("*    Mark-5 data time monitor                      *\n");
       printf("*      %s               *\n",PROGRAM_VERSION);
       printf("*          Kashima Space Research Center/NICT      *\n");
       printf("****************************************************\n");
       printf("\n");

       /* run parameter get */
       param_get_m5time(argc, argv, mk5name, &bitpos, &ntrack, &nkai, &vlba, &sbyte);

       if (mk5name[0]=='\0'){
          strcpy(mk5name,dflt_fname);
       }

       /* Mark5 file existence check */
       if( (mk5.fin  = fopen( mk5name, "rb" )) == NULL ){
           printf("Mark5 File %s   Open Error! \n",mk5name);
		   goto exitend;
       }
       i=fclose(mk5.fin);



       printf("Mark 5 Data File : %s\n",mk5name);

       printf("ntrack= %d  bitpos = %d\n",ntrack,bitpos);
       if(vlba==1){
           mk5.vlba=1;
           mk5.nrzm=1;
           mk5.parity=1;
           printf("VLBA mode\n");
       } else {
           mk5.vlba=0;
           mk5.nrzm=0;
           mk5.parity=0;
           printf("Mark4 mode\n");
       }
       if(vlba==0){ // Mark-IV format
 	       printf("       HEADER     AUX     SYNC    TIME FIELD  CRC  \n");
 	       printf("    # aaaabbbb rrssttuu          YDDDHHMM SSsss 12\n");
       } else {  // VLBA format
           printf("       HEADER     AUX     SYNC    TIME CODE   CRC \n");
 	       printf("    # SSRHPPPP TGBSCD00          MJDsssss.ssss  16\n");
       } 
       mk5.bitpos=bitpos;
       mk5.ko=0;
       mk5.ntrack=ntrack;

           /* Data file open */
       if( (mk5.fin  = fopen( mk5name, "rb" )) == NULL ){
              printf("Data File %s   Open Error! \n",mk5name);
		      goto exitend;
       }

	       /* skip 1st sbyte */
       fseek(mk5.fin,sbyte,SEEK_CUR); // move pointer by sbyte
       if( ferror( mk5.fin ) ){
	        i=fclose(mk5.fin);
 	        goto exitend;
	   }

       //for(n=0; n<nkai; n++){
	   n=0;
	   while(1){
          //i=read_1frame_r(&mk5);
          i=read_1frame_r_mugen(&mk5);

	      if(i<=0) {
		     i=fclose(mk5.fin);
		     goto exitend;
		  }

          printf("%5d",n);
          for (i=0; i<5; i++){   //check           
            printf(" %08X",rvsint(mk5.header[i]));
          } 
          printf("\n");
          //printf("%d %d %d\n",n,mk5.aux.trk,mk5.aux.adid); //debug
		  if(nkai > 0 && n == nkai-1) break;
		  n++;
       }
	   
	   i=fclose(mk5.fin);

exitend: 
       return 0;
	   
}
